package extraction;

import org.apache.commons.lang.StringUtils;
import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.drools.RuleBase;
import org.drools.RuleBaseFactory;
import org.drools.compiler.PackageBuilder;
import org.drools.rule.Package;

import java.io.InputStreamReader;
import java.io.Reader;
import java.io.InputStream;
import java.util.ArrayList;
import java.util.Collection;
import java.util.List;

public class RuleParser {

    /**
     * Logging object
     */
    private static Log log = LogFactory.getLog(RuleParser.class);

    /**
     * Singleton object
     */
    private static RuleParser instance;


    /**
     * Drools RuleBase object.
     */
    private RuleBase ruleBase;

    /**
     * Constructor
     *
     * @throws RuleException Rule exception
     */
    private RuleParser() throws RuleException {
        init();
    }


    /**
     * Initialization.
     *
     * @throws RuleException Rule exception.
     */
    private void init() throws RuleException {
        try {
            AppConfiguration config = AppConfiguration.getInstance();
            Object obj = config.getProperty("ruleset.rule-file");
            List<String> ruleList = null;
            if (obj instanceof Collection) {
                int size = ((Collection) obj).size();
                ruleList = new ArrayList<String>(size);
                for (int i = 0; i < size; i++) {
                    String ruleFile = config.getString("ruleset.rule-file(" + i + ")");
                    if (StringUtils.isNotBlank(ruleFile))
                        ruleList.add(ruleFile);
                }
            } else if (obj instanceof String) {
                ruleList = new ArrayList<String>(1);
                String ruleFile = config.getString("ruleset.rule-file");
                if (StringUtils.isNotBlank(ruleFile))
                    ruleList.add(ruleFile);
            }

            // Rule package builder
            PackageBuilder builder = new PackageBuilder();

            // Add the package to a rulebase (deploy the rules package).
            ruleBase = RuleBaseFactory.newRuleBase();

            if (ruleList != null && ruleList.size() > 0) {
                for (String ruleFile : ruleList) {
                    if (log.isDebugEnabled()) {
                        log.debug("Loading rules from  " + ruleFile);
                    }
                    // Read in the source
                    InputStream in = RuleParser.class.getResourceAsStream(ruleFile);
                    Reader source = new InputStreamReader(RuleParser.class.getResourceAsStream(ruleFile));

                    // This wil parse and compile in one step
                    builder.addPackageFromDrl(source);

                    source.close();
                }
                // Get the compiled package (which is serializable)
                Package pkg = builder.getPackage();
                ruleBase.addPackage(pkg);
            }
        } catch (Exception ex) {
            if (log.isErrorEnabled()) {
                log.error("Error loading rules. " + ex.getMessage());
                ex.printStackTrace();
            }
            throw new RuleException(ex.getMessage(), ex);
        }
    }


    /**
     * Singleton access method.
     *
     * @return Singleton object
     * @throws RuleException Rule exception.
     */
    public static RuleParser getInstance() throws RuleException {
        if (instance == null)
            instance = new RuleParser();
        return instance;
    }

    /**
     * Get rule base
     *
     * @return Rule base
     */
    public RuleBase getRuleBase() {
        return ruleBase;
    }

    /**
     * Set rule base
     *
     * @param ruleBase Rule base
     */
    public void setRuleBase(RuleBase ruleBase) {
        this.ruleBase = ruleBase;
    }

    /**
     * Reload the rule engine, and recompile all the rule files
     *
     * @throws RuleException Rule exception
     */
    public static synchronized void reload() throws RuleException {
        RuleParser newInstance = new RuleParser();
        RuleParser oldInstance = instance;
        instance = newInstance;
        oldInstance = null;
    }

}

